@@ -13,13 +13,20 @@ from typing import (
     Optional,
     Sequence,
     Tuple,
+    Type,
     TypeVar,
     Union,
     cast,
-    get_type_hints,
     overload,
 )
 
+try:
+    # Python 3.7+
+    from typing_extensions import get_type_hints  # type: ignore[attr-defined]
+except ImportError:
+    # Python 3.6
+    from typing import get_type_hints  # type: ignore[misc]  # incompatible import
+
 # Literal
 if sys.version_info >= (3, 8):
     from typing import Literal  # Python 3.8+
@@ -137,6 +144,13 @@ _SimpleTypeVarCo = TypeVar("_SimpleTypeVarCo", covariant=True)
 _MISSING = object()
 _FAILURE = object()
 
+if TYPE_CHECKING:
+    from typing_extensions import TypeAlias, TypeGuard
+
+    TypeGuard_T: TypeAlias = TypeGuard[_T]
+else:
+    TypeGuard_T = bool
+
 
 # TODO: Use this signature for trycast once support for TypeForm is
 #       implemented in mypy.
@@ -147,16 +161,18 @@ _FAILURE = object()
 
 
 @overload
-def trycast(tp: object, value: object) -> Optional[object]:
+def trycast(tp: object, value: object, *, strict: bool = False) -> Optional[object]:
     ...
 
 
 @overload
-def trycast(tp: object, value: object, failure: _F) -> Union[object, _F]:
+def trycast(
+    tp: object, value: object, failure: _F, *, strict: bool = False
+) -> Union[object, _F]:
     ...
 
 
-def trycast(tp, value, failure=None):
+def trycast(tp, value, failure=None, *, strict=False):
     """
     If `value` is in the shape of `tp` (as accepted by a Python typechecker
     conforming to PEP 484 "Type Hints") then returns it, otherwise returns
@@ -183,6 +199,18 @@ def trycast(tp, value, failure=None):
     also be a valid float value, as consistent with Python typecheckers:
         > trycast(float, 1) -> 1
         > isinstance(1, float) -> False
+
+    If strict=False then trycast will additionally accept
+    mypy_extensions.TypedDict instances and Python 3.8 typing.TypedDict
+    instances for the `tp` parameter. Normally these kinds of types are
+    rejected by trycast with a TypeNotSupportedError because these
+    types do not preserve enough information at runtime to reliably
+    determine which keys are required and which are potentially-missing.
+
+    Raises:
+    * TypeNotSupportedError --
+        If strict=True and either mypy_extensions.TypedDict or a
+        Python 3.8 typing.TypedDict is found within the `tp` argument.
     """
     if tp is int:
         # Do not accept bools as valid int values
@@ -275,10 +303,21 @@ def trycast(tp, value, failure=None):
                 required_keys = tp.__required_keys__
             except AttributeError:
                 # {typing in Python 3.8, mypy_extensions}.TypedDict
-                if tp.__total__:
-                    required_keys = resolved_annotations.keys()
+                if strict:
+                    if sys.version_info[:2] >= (3, 9):
+                        advise = "Suggest use a typing.TypedDict instead."
+                    else:
+                        advise = "Suggest use a typing_extensions.TypedDict instead."
+                    raise TypeNotSupportedError(
+                        "trycast cannot determine which keys are required "
+                        "and which are potentially-missing for the "
+                        "specified kind of TypedDict. " + advise
+                    )
                 else:
-                    required_keys = frozenset()
+                    if tp.__total__:
+                        required_keys = resolved_annotations.keys()
+                    else:
+                        required_keys = frozenset()
 
             for (k, v) in value.items():
                 V = resolved_annotations.get(k, _MISSING)
@@ -304,6 +343,10 @@ def trycast(tp, value, failure=None):
         return failure
 
 
+class TypeNotSupportedError(ValueError):
+    pass
+
+
 def _trycast_listlike(
     tp, value, failure, listlike_type, *, covariant_t=False, t_ellipsis=False
 ):
@@ -368,3 +411,55 @@ def _is_simple_typevar(T: object, covariant: bool = False) -> bool:
         and T.__contravariant__ is False
         and T.__constraints__ == ()
     )
+
+
+# TODO: Use this signature for isassignable once support for TypeForm is
+#       implemented in mypy. See: https://github.com/python/mypy/issues/9773
+# @overload
+# def isassignable(value: object, tp: TypeForm[_T]) -> TypeGuard_T: ...
+
+
+@overload
+def isassignable(value: object, tp: Type[_T]) -> TypeGuard_T:
+    ...
+
+
+@overload
+def isassignable(value: object, tp: object) -> bool:
+    ...
+
+
+def isassignable(value, tp):
+    """
+    Returns whether `value` is in the shape of `tp`
+    (as accepted by a Python typechecker conforming to PEP 484 "Type Hints").
+
+    This method logically performs an operation similar to:
+
+        return isinstance(tp, value)
+
+    except that it supports many more types than `isinstance`, including:
+        * List[T]
+        * Dict[K, V]
+        * Optional[T]
+        * Union[T1, T2, ...]
+        * Literal[...]
+        * T extends TypedDict
+
+    Note that unlike isinstance(), this method does NOT consider bool values
+    to be valid int values, as consistent with Python typecheckers:
+        > isassignable(False, int) -> False
+        > isinstance(False, int) -> True
+
+    Note that unlike isinstance(), this method considers every int value to
+    also be a valid float value, as consistent with Python typecheckers:
+        > isassignable(1, float) -> True
+        > isinstance(1, float) -> False
+    """
+    if trycast(tp, value, _isassignable_failure, strict=True) is _isassignable_failure:
+        return False
+    else:
+        return True
+
+
+_isassignable_failure = object()
